#import "ARMGLESView.h"

#include "main.h"


@implementation ARMGLESView

- (id)initWithFrame:(NSRect)frame
{
    if ((self = [super initWithFrame:frame])) 
    {
				_glesClient = new GLESClient(self);	

				_start = [[NSDate date] retain];	

        /* 
         * The ARMGLESView relies on Cocoa to call us back to draw the 
         * contents of the window, so set up a 60Hz timer that will be 
         * responsible for triggering the call to drawRect:.
         */
        NSTimer *timer = [NSTimer scheduledTimerWithTimeInterval:1/60.0 
                                                          target:self
                                                        selector:@selector(timerEvent:) 
                                                        userInfo:NULL
                                                         repeats:YES];
        
        [[NSRunLoop currentRunLoop] addTimer:timer 
                                     forMode:NSDefaultRunLoopMode];

    }
    
    return self;
}

- (void)timerEvent:(id)sender
{
    [self setNeedsDisplay:YES];
}

-(NSString*)getShaderSourceFromFile:(NSString *) filename{ 

    /* Load in the shader source code */
    NSError *error = [[NSError alloc] init];
    NSString *shaderSource = [NSString stringWithContentsOfFile:filename
                                                       encoding:NSUTF8StringEncoding
                                                          error:&error];
    
    /* Check we could load the file */
    if (shaderSource == nil)
    {
        [NSApp presentError:error];
				return nil;
    }
    
    /* Release the error object */
    [error release];

		return shaderSource;
}

- (void)prepare
{

		/* Process shaders */
    NSString *vertexSource   = [self getShaderSourceFromFile:[[NSBundle mainBundle] pathForResource:@"shader" 
                                                               ofType:@"vert"]];
    NSString *fragmentSource = [self getShaderSourceFromFile:[[NSBundle mainBundle] pathForResource:@"shader" 
                                                               ofType:@"frag"]];

		if(!vertexSource || !fragmentSource){
			NSLog(@"Could not load shaders");
			return;
		}
		_glesClient->setVertexShaderSource([vertexSource UTF8String]);
		_glesClient->setFragmentShaderSource([fragmentSource UTF8String]);

		_glesClient->init();
}

- (void)cleanup
{
    /* Shut down EGL */
    EGL_CHECK(eglMakeCurrent(_display, EGL_NO_SURFACE, EGL_NO_SURFACE, EGL_NO_CONTEXT));
    EGL_CHECK(eglDestroyContext(_display, _context));
    EGL_CHECK(eglDestroySurface(_display, _surface));
    EGL_CHECK(eglTerminate(_display));

		delete _glesClient;
		_glesClient = 0;
}

- (void)drawRect:(NSRect)dirtyRect
{
    /* Initialise EGL if necessary */
    if (!_eglInitialized)
    {
        [self prepare];
        
        _eglInitialized = YES;
    }

		_glesClient->setElapsedTime(-[_start timeIntervalSinceNow]);		

		_glesClient->cycle();
		
		_glesClient->draw();
		return;

		//Call the main draw cycle
		//draw();

/*
		glVertextAttribPointer(_colour, 4, GL_FLOAT, GL_FALSE,
														sizeof(Vertex), (GLvoid*) (sizeof(float) * 3));
*/
    
    /* Swap the buffers and draw the scene to the view */
    EGL_CHECK(eglSwapBuffers(_display, _surface));
}

@end
